#!/usr/bin/env python3

import argparse
import os
import sys

from rich.console import Console
from rich.table import Table

from . import __version__
from .size import FontSize


def main():  # pragma: no cover
    run(sys.argv[1:])


def run(argv):
    # rich package console objects
    console = Console()
    error_console = Console(stderr=True)
    # ===========================================================
    # argparse command line argument definitions
    # ===========================================================
    parser = argparse.ArgumentParser(
        description="Font file and OpenType table size reporting tool"
    )
    parser.add_argument(
        "--version", action="version", version="font-size v{}".format(__version__)
    )
    parser.add_argument("FONTPATHS", help="One or more font file paths", nargs="*")

    args = parser.parse_args(argv)

    for fontpath in args.FONTPATHS:
        # validations
        if not os.path.exists(fontpath):
            error_console.print(
                f"[bold red][ERROR][/] {fontpath} does not appear to be a valid "
                f"file path. Please enter a path to a font and try again."
            )
            sys.exit(1)

        # instantiate FontSize object
        # this calculates file size data with a fontTools TTFont obj
        try:
            fontsize = FontSize(fontpath)
        except Exception as e:
            error_console.print(
                f"[bold red][ERROR][/] Attempt to read {fontpath} raised the "
                f"following error: {e}"
            )
            sys.exit(1)

        print()
        console.print(f"[green]{fontpath}[/]\n")

        table = Table(
            title="Summary Statistics", show_header=True, header_style="bold magenta"
        )
        table.add_column("Statistic", style="dim", justify="right")
        table.add_column("Size (B)", justify="right")
        table.add_row(
            "Total",
            f"{fontsize.total_bytes/1.0:.2f}",
        )
        table.add_row(
            f"Avg per encoded glyph ({fontsize.encoded_glyphs})",
            f"{fontsize.per_enc_glyph_bytes:.2f}",
        )
        table.add_row(
            f"Avg per total glyph ({fontsize.total_glyphs})",
            f"{fontsize.per_total_glyph_bytes:.2f}",
        )
        console.print(table)

        tt_tables = fontsize.get_table_tags()

        table_ottables = Table(
            title="OpenType Table Sizes",
            show_header=True,
            header_style="bold magenta",
            show_lines=True,
        )
        table_ottables.add_column(
            "OpenType Table", style="dim", width=25, justify="right"
        )
        table_ottables.add_column("Size (B)", justify="right")
        table_ottables.add_column("% Total", justify="right")
        for tt_table in sorted(tt_tables):
            size = fontsize.tables[tt_table]
            percent_size = (size / fontsize.total_bytes) * 100
            table_ottables.add_row(f"{tt_table}", f"{size}", f"{percent_size:.3f}")
        print()
        console.print(table_ottables)



#!/usr/bin/env python3

import os

from fontTools.ttLib import TTFont


class FontSize(object):
    def __init__(self, filepath):
        self.ttfont = TTFont(filepath)
        self.filepath = filepath
        self.total_glyphs = 0
        self.encoded_glyphs = 0
        self.total_bytes = 0
        self.per_enc_glyph_bytes = 0.0
        self.per_total_glyph_bytes = 0.0
        self.tables = {}
        self._calculate_sizes()

    def _calculate_sizes(self):
        self.total_bytes = os.path.getsize(self.filepath)
        self._calc_per_encoded_glyph_size()
        self._calc_per_total_glyph_size()
        self._calc_table_sizes()

    def _calc_per_encoded_glyph_size(self):
        self.encoded_glyphs = len(self.ttfont.getBestCmap())
        self.per_enc_glyph_bytes = self.total_bytes / self.encoded_glyphs

    def _calc_per_total_glyph_size(self):
        self.total_glyphs = len(self.ttfont.getGlyphSet())
        self.per_total_glyph_bytes = self.total_bytes / self.total_glyphs

    def _calc_table_sizes(self):
        tables = self.get_table_tags()
        for table in tables:
            self.tables[table] = self.ttfont.reader.tables[table].length

    def get_table_tags(self):
        return [tag for tag in self.ttfont.keys() if tag != "GlyphOrder"]



import io
import os
import sys

from setuptools import find_packages, setup

# Package meta-data.
NAME = "font-size"
DESCRIPTION = "Font file and OpenType table size reporting tool"
LICENSE = "Apache License v2.0"
URL = "https://github.com/source-foundry/font-size"
EMAIL = "chris@sourcefoundry.org"
AUTHOR = "Source Foundry Authors"
REQUIRES_PYTHON = ">=3.6.0"

INSTALL_REQUIRES = ["fontTools", "brotli", "rich"]
# Optional packages
EXTRAS_REQUIRES = {
    # for developer installs
    "dev": ["coverage", "pytest", "tox", "flake8", "mypy", "black"],
    # for maintainer installs
    "maintain": ["wheel", "setuptools", "twine"],
}

this_file_path = os.path.abspath(os.path.dirname(__file__))

# Version
main_namespace = {}
version_fp = os.path.join(this_file_path, "lib", "fontsize", "__init__.py")
try:
    with io.open(version_fp) as v:
        exec(v.read(), main_namespace)
except IOError as version_e:
    sys.stderr.write(
        "[ERROR] setup.py: Failed to read data for the version definition: {}".format(
            str(version_e)
        )
    )
    raise version_e

# Use repository Markdown README.md for PyPI long description
try:
    with io.open("README.md", encoding="utf-8") as f:
        readme = f.read()
except IOError as readme_e:
    sys.stderr.write(
        "[ERROR] setup.py: Failed to read README.md for the long description: {}".format(
            str(readme_e)
        )
    )
    raise readme_e

setup(
    name=NAME,
    version=main_namespace["__version__"],
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    license=LICENSE,
    platforms=["Any"],
    long_description=readme,
    long_description_content_type="text/markdown",
    package_dir={"": "lib"},
    packages=find_packages("lib"),
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRES,
    python_requires=REQUIRES_PYTHON,
    entry_points={"console_scripts": ["font-size = fontsize.__main__:main"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
